// file: lat_cstr_0.cc
//

// isip include files
//
#include "lattice.h"
#include "lattice_constants.h"

// method: destructor
//
// arguments: none
//
// return: none
//
// this is the default destructor
//
Lattice::~Lattice() {

  // dummy variables
  //
  Lattice_node* ltn = (Lattice_node*)NULL;
  Hash_cell* next = (Hash_cell*)NULL;

  // memory manager
  //
  Memory_manager* manager = Link_list::get_manager_cc();

  // delete lattice node pointers
  //
  Hash_cell** hcells = lnode_d->get_cells_cc();
  int_4 hash_size = lnode_d->get_size_cc();
  for (int_4 k = 0; k < hash_size; k++) {
    for (Hash_cell* lcell = hcells[k]; lcell != (Hash_cell*)NULL;
         lcell = next) {
      next = lcell->get_next_cc();
      ltn = (Lattice_node*)(lcell->get_item_cc());
      ltn->clear_cc();
      manager->delete_cc(ltn);
      manager->delete_cc(lcell);
    }
    hcells[k] = (Hash_cell*)NULL;
  }
  delete lnode_d;
  
  // delete start node of the lattice
  //
  if (start_node_d != (Lattice_node*)NULL) {
    start_node_d = (Lattice_node*)NULL;
  }
  
  // delete lattice generation information
  //
  null_word_d = (Word*)NULL;
  if (lang_model_d != (char_1*)NULL) {
    delete [] lang_model_d;
    lang_model_d = (char_1*)NULL;
  }
  if (utterance_d != (char_1*)NULL) {
    delete [] utterance_d;
    utterance_d = (char_1*)NULL;
  }
  if (model_set_d != (char_1*)NULL) {
    delete [] model_set_d;
    model_set_d = (char_1*)NULL;
  }
  if (name_d != (char_1*)NULL) {
    delete [] name_d;
    name_d = (char_1*)NULL;
  }
  
  // reset likelihood scale factors
  //
  lm_scale_d = LATTICE_DEFAULT_SCALE;
  word_penalty_d = LATTICE_DEFAULT_PENALTY;
  
  // reset node statistics
  //
  num_arcs_d = LATTICE_DEFAULT_NUM_ARCS;
  num_nodes_d = LATTICE_DEFAULT_NUM_NODES;

  // exit gracefully
  //
}
