// file: comm_frontend_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "extract_feature.h"

// Communicator API
//
#include "communicator.h"

Extract_feature frontend;

// method: init_frontend
//
// arguments: none
//
// return: a logical flag to indicate status
//
// this method initilizes the frontend from parameter file
//
int init_frontend(char* params_a) {

  // read the parameter files
  //
  if (frontend.read_parameter_cc((char_1*)params_a) == ISIP_FALSE) {
    
    // output error message and exit
    //
    fprintf(stderr, "Extract_feature : Error reading command line parameters\n");
    
    exit(ISIP_PROTO_ERROR);
  }

  // configure the frontend
  //
  frontend.config_frontend_cc();
	
  // return gracefully
  //
  return ISIP_TRUE;
}

// method: uttproc_rawdata
//
// arguments:
//  char* data: (input) feature input buffer
//  int* num_samples: (input) number of feature data
//  int block: (input) pending blocks (currently unused)
//
// return: bool indicating status
//
// this method decodes an utterance specified by the input mfc data.
//
int uttproc_rawdata(short* in_data, int num_samples, int block) {

  // local variables
  //
  int_4 samples = num_samples;
  
  // compute the feature data from audio raw data
  //
  float_8* mfc_data = frontend.compute_cc((int_2*&)in_data, samples,
					  (int_4)block);

  // decoding procedure
  //
  if (mfc_data != NULL) {
    uttproc_mfcdata((double*)mfc_data, samples);
    delete [] mfc_data;
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}

// method: clean_frontend
//
// arguments:
//  none
//
// return: bool indicating status
//
// this method clean up the frontend before receiving a new utterance
//
int clean_frontend() {
  return frontend.clean_cc();
}
